/*
  focus.h - laser focus module header
    Part of TwinTeeh Firmware

Copyright (c) 2015 V.Ayllon.  All rights reserved.

TwinTeeth Firmware is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
to anyone for the consequences of using it or for whether it serves any
particular purpose or works at all, unless he says so in writing.  Refer
to the GNU General Public License, Version 2 or later, for full details.

Everyone is granted permission to copy, modify and redistribute
TwinTeeth Firmware, but only under the conditions described in the GNU General Public
License.  A copy of this license is supposed to have been
given to you along with TwinTeeth Firmware so you can know your rights and
responsibilities.  It should be in a file named COPYING.  Among other
things, the copyright notice and this notice must be preserved on all
copies.

- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/

#ifndef	FOCUS_H
#define	FOCUS_H

#define MIN_FOCUS_VALUE	   60   // Min focus value

#define FOCUS_ST_DONE		0	// The laser is focused on the PCB
#define FOCUS_ST_RESET		1	// Init and reset state
#define FOCUS_MAX_TRIES		4	// MAx tries to reset and focus again

#define NUM_FOCUS_POINTS		4	// Number of focus points (don�t change)

#define BLUE_LASER_FOCUS_POWER   115  //
#define IR_LASER_FOCUS_POWER     150  

#define FOCUS_ERROR			0xff
#define FOCUS_OK			0

// FE values

#define MIN_FE_VALUE	0		//Min focus error signal val 
#define MAX_FE_VALUE	1023	 //Max focus error signal val
#define REF_FE_VALUE	505		//Typical Vref digital value

// Focus Servo

#define FSERVO_MIN       0	                        // Focus servo min position (inside pickup)
#define FSERVO_MAX       255	                        // Focus servo max position (outside)
#define FSERVO_CENTER    128
#define MAX_FOCUS_SWEEP  6                              // Max focus attempts
#define NUM_FOCUS_SAMPLES  10   // Num focus samples
#define FOCUS_TIMEOUT    30000                          // Focus timeout in ADC ticks at 2000Khz.

#define MIN_AMP_THRESHOLD  30 // 65  // No object limit
#define FE_NOISE_DIVIDER   3   // Used to find the noise threshold
#define AMPLITUD_DIVIDER   8  // 48 Amplitud/(this) is the focus threshold 

#define MAX_FE_DATA       290

#define FE_WAIT_ST         0
#define FE_AMPLITUD_ST     1
#define FE_FOCUS_ST        2
#define FE_SCURVE_ST       3
#define FE_FOCUSED_ST      4
#define FE_ERROR_ST        5      

// FE values

void set_focus_diode(uint8_t d);
uint8_t auto_focus();
uint8_t get_focus_position();
void start_fe_sampler();
void stop_fe_sampler();
void set_state(uint8_t state);
uint8_t calc_mode(uint8_t *value, uint8_t num);
void init_focus(float foffset, float width, float height);
uint8_t get_focus_points();
uint8_t interpolate(float x, float y);

extern float focus_offset;

#endif
